/*
 * System Interface Library for games
 * Copyright (c) 2011 Andrew Church <achurch@achurch.org>
 * Released under the GNU GPL version 3 or later; NO WARRANTY is provided.
 * See the file COPYING.txt for details.
 *
 * src/test/sysdep/windows/time.c: Tests for Windows-specific timekeeping
 * functions.
 */

#include "src/base.h"
#define IN_SYSDEP  // So we get the real functions instead of the diversions.
#include "src/sysdep.h"
#include "src/sysdep/windows/internal.h"
#include "src/test/base.h"

/*************************************************************************/
/****************************** Test runner ******************************/
/*************************************************************************/

DEFINE_GENERIC_TEST_RUNNER(test_windows_time)

TEST_INIT(init)
{
    LARGE_INTEGER frequency_buf;
    QueryPerformanceFrequency(&frequency_buf);
    int64_t frequency = frequency_buf.QuadPart;
    while (frequency >= INT64_C(1) << 31) {
        frequency >>= 1;
    }
    CHECK_INTEQUAL(sys_time_init(), frequency);
    return 1;
}

/*************************************************************************/
/***************************** Test routines *****************************/
/*************************************************************************/

TEST(test_delay)
{
    LARGE_INTEGER frequency_buf;
    QueryPerformanceFrequency(&frequency_buf);
    const int64_t true_frequency = frequency_buf.QuadPart;
    const int64_t true_ticks_per_csec = true_frequency / 100;
    int64_t frequency = true_frequency;
    while (frequency >= INT64_C(1) << 31) {
        frequency >>= 1;
    }
    const int64_t ticks_per_csec = frequency / 100;

    LARGE_INTEGER start, end;

    QueryPerformanceCounter(&start);
    QueryPerformanceCounter(&end);
    CHECK_INTRANGE(end.QuadPart - start.QuadPart, 0, true_ticks_per_csec/2);

    QueryPerformanceCounter(&start);
    sys_time_delay(10*ticks_per_csec);
    QueryPerformanceCounter(&end);
    /* Sleep() seems to sometimes wake up slightly before the requested
     * amount of time has passed. */
    CHECK_INTRANGE(end.QuadPart - start.QuadPart,
                   9*true_ticks_per_csec, 12*true_ticks_per_csec);

    QueryPerformanceCounter(&start);
    sys_time_delay(0);
    QueryPerformanceCounter(&end);
    CHECK_INTRANGE(end.QuadPart - start.QuadPart, 0, true_ticks_per_csec/2);

    return 1;
}

/*************************************************************************/
/*************************************************************************/

/*
 * Local variables:
 *   c-file-style: "stroustrup"
 *   c-file-offsets: ((case-label . *) (statement-case-intro . *))
 *   indent-tabs-mode: nil
 * End:
 *
 * vim: expandtab shiftwidth=4:
 */
